<?php
/**
* Convert all GIF files in the posters directory to PNG.
*
* $Id: gif2pnglib.php,v 1.1 2003/12/31 14:03:33 niko Exp $
*/

/**
* convertGifs: gets gifs from database and kicks off conversion.
*
* @param int     If you want to convert a special database id (e.g. new entry only)
* @param string  dir where the gifs are stored. If nothing specified use LocalPosterPath
* @param string  name of the converter executable (you may have to change the path here
                 default is /usr/local/bin/gif2png
* @return int    Number of converted files.
*/
function convertGifs ($id = "", $gifDir ="", $gifExec="") {
  global $DEBUG, $VERBOSE, $CFG;
  $gifs = array();
  
  if (!$gifDir) $gifDir = $CFG["LocalPosterPath"];
  if ($VERBOSE) echo "getting gif(s) in poster dir " . $gifDir . " ...<br>";

  $strSql = "SELECT ID, LocalPic FROM ". $CFG['Prefix'] . "movies WHERE LocalPic LIKE '%gif'";
  if ($id) { $strSql .= " AND ID=$id"; }
  $sqlGifs = mysql_query($strSql, $pmldb);
  $i = 0;
  while($row = mysql_fetch_array($sqlGifs)) {
    $newfiles = changeGifs($row["LocalPic"], $gifDir, $gifExec);
    if (sizeof($newfiles)>0) {
      $strSql = "UPDATE ". $CFG['Prefix'] ."movies SET LocalPic ='"
                . $newfiles[0] ."' WHERE ID=". $row['ID'];
      if ($DEBUG) { echo "DEBUG: " . $strSql . "<br>"; }
      else { $sqlNew = mysql_query($strSql, $pmldb); }
      $i++;

      if (! preg_match("/.jpg$/",$newfiles[0])) removeGifs($row["LocalPic"],$gifDir);
    }
  }
  if ($VERBOSE && !$DEBUG) echo $i . " images converted or renamed.<br>";
  if ($DEBUG) echo "no images converted (DEBUG mode)<br>"; 
  return $i;
}

/**
* changeGifs: gets a list of files and does the conversion.
*
* If a file is a normal gif do the conversion with gif2png (check path if you 
* get errors) If gif2png says this is not a GIF, it probably is a JPEG/JFIF
* so we'll rename it.
*
* @param array string  gif filenames.
* @param string  dir where the gifs are stored. If nothing specified use LocalPosterPath
* @param string  name of the converter executable (you may have to change the path here
                 default is /usr/local/bin/gif2png
* @return string[] List of converted files (new filenames).
*/
function changeGifs($gifs, $gifDir="", $gifExec = "") {
  global $DEBUG, $VERBOSE, $CFG;
  $workingGifs = array();
  
  if (!$gifDir) $gifDir = $CFG["LocalPosterPath"];
  if (!$gifExec) $gifExec = "/usr/local/bin/gif2png";
  if ($VERBOSE) echo "converting ". sizeof($gifs) ." gif(s) ...";
  
  $options = "-O"; //optimize png
  if (! IsWriteable($gifDir)) ErrorExit("Directory $gifDir is not writeable.<br>");
  // can be called with a single file
  if (! is_array($gifs)) $gifs = array($gifs);


  while ($g = array_pop($gifs)) {
    if ($DEBUG) {
      echo "DEBUG: " . $gifExec . " " . $options . " $g ";
      $ret_val = 0;
    } else {
      $output = @system($gifExec . " " . $options . " " 
                       .$gifDir ."/" . $g . " 2>&1", $ret_val);
    }
    if ($ret_val != 0) {
      echo "An error occured while converting file "
           . $gifDir . "/" .  $g . " with gif2png "
           ."($gifExec). Output <br><pre>$output</pre>";
      // this is most likely a jpeg with wrong extension.
      if (strstr($output, "not a GIF")) {
        $newfile = preg_replace("/\.gif/",".jpg",$g);
        if ($DEBUG) {
          echo "DEBUG: renaming $gifDir/$g to $gifDir/$newfile (this is a jpeg)<br>";
        } else { 
          if ($VERBOSE) echo "renaming $g to $newfile (this is a jpeg)<br>";
	  rename($gifDir . "/" . $g, $gifDir . "/" . $newfile);
	}
        array_push($workingGifs, $newfile);
      }
    } else {
      $newfile = preg_replace("/\.gif/",".png",$g);
      array_push($workingGifs, $newfile);
    }
  }
  if ($VERBOSE) echo "done.<br>";
  return $workingGifs;
}
/**
* removeGifs: gets a list of files and deletes them
*
* @param array string  gif filenames.
* @param string  dir where the gifs are stored. If nothing specified use LocalPosterPath
* @return boolean okFlag: false if an error appeared while deleting. true if all is ok.
*/
function removeGifs ($gifs, $gifDir = "") {
  global $DEBUG, $VERBOSE, $CFG;
  $okFlag = true;

  if (!$gifDir) $gifDir = $CFG["LocalPosterPath"];
  if ($VERBOSE) echo "deleting ". sizeof($gifs) ." old gif(s)..." ;
  
  if (! IsWriteable($gifDir)) ErrorExit("Directory $gifDir is not writeable.<br>");
  // can be called with a single file
  if (! is_array($gifs)) $gifs = array($gifs);

  for ($i = 0; $i < sizeof($gifs);$i++) {
    if  ($DEBUG) {
      echo "DEBUG: unlinking $gifDir/$gifs[$i]<br>";
      $retFlag = true;
    } else {
      $retFlag = unlink ($gifDir . "/" . $gifs[$i]);
    }
    if (!$retFlag) {
      $okFlag = false;
      if ($VERBOSE) echo "An error occured while deleting old gif file ($gifDir/$gifs[$i])<br>";
    }
  }
  if ($VERBOSE) echo "done.<br>";
  return $okFlag;
}

?>
